/*-----------------------------------------------------------------------

File  : clb_ddarrays.h

Author: Stephan Schulz

Contents
 
  Dynamic arrays of large data types - at the moment doubles only.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Sun Aug  8 22:45:29 GMT 1999
    Copied from clb_pdarrays.h

-----------------------------------------------------------------------*/

#ifndef CLB_DDARRAYS

#define CLB_DDARRAYS

#include <clb_memory.h>


/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef struct ddarraycell
{
   long   size;
   long   grow;
   double *array;
}DDArrayCell, *DDArray_p; 


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

#define DDArrayCellAlloc() (DDArrayCell*)SizeMalloc(sizeof(DDArrayCell))
#define DDArrayCellFree(junk) SizeFree(junk, sizeof(DDArrayCell))

DDArray_p DDArrayAlloc(long init_size, long grow);
void      DDArrayFree(DDArray_p junk);

void      DDArayEnlarge(DDArray_p array, long index);
static __inline__ double* DDArrayElementRef(DDArray_p array, long index);


#define   DDArrayAssign(array, index, value) \
          *DDArrayElementRef((array), (index)) = (value)

#define   DDArrayElement(array, index) \
	  *DDArrayElementRef((array), (index))

void      DDArrayAdd(DDArray_p collect, DDArray_p data, long limit);

double    DDArraySelectPart(DDArray_p array, double part, long size);


/*---------------------------------------------------------------------*/
/*                     Inline functions                                */
/*---------------------------------------------------------------------*/

	  
/*-----------------------------------------------------------------------
//
// Function: DDArrayElementRef()
//
//   Return a reference to an element in a dynamic array. This
//   reference is only good until the next call to this function! User
//   programs are expected to use this function only extremely rarely
//   and with special care. Use DDArrayElement()/DDArrayAssign()
//   instead. 
//
// Global Variables: -
//
// Side Effects    : May enlarge and move array.
//
/----------------------------------------------------------------------*/

static __inline__ double* DDArrayElementRef(DDArray_p array, long index)
{
   assert(array);
   assert(index >= 0);
   
   if(!(index < array->size))
   {
      DDArayEnlarge(array, index);
   }
   return &(array->array[index]);
}

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





