/*-----------------------------------------------------------------------

File  : cco_eqnresolving.c

Author: Stephan Schulz

Contents
 
  Functions controlling equality resolution.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Mon Jun  8 17:17:57 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "cco_eqnresolving.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: ComputeAllEqnResolvents()
//
//   Compute all equality resolvents of clause and put them into store.
//
// Global Variables: -
//
// Side Effects    : As above
//
/----------------------------------------------------------------------*/

long ComputeAllEqnResolvents(TB_p bank, Clause_p clause, ClauseSet_p
			     store, VarBank_p freshvars, long *idents)
{
   Clause_p    resolvent;
   Eqn_p       test;
   ClausePos_p pos;
   long        resolv_count = 0;

   if(clause->neg_lit_no)
   {
      pos = ClausePosAlloc();
      
      test = ClausePosFirstEqResLiteral(clause, pos);
      
      while(test)
      {
	 resolvent = ComputeEqRes(bank, pos, freshvars);
	 if(resolvent)
	 {
	    resolv_count++;
	    resolvent->ident = ++(*idents);
	    resolvent->parent1 = clause;
	    resolvent->proof_depth = clause->proof_depth+1;
	    resolvent->proof_size  = clause->proof_size+1;
	    ClauseSetTPTPType(resolvent, ClauseQueryTPTPType(clause));
	    ClauseRegisterChild(clause, resolvent);
	    ClauseDefaultDocumentCreation(resolvent, CC_ERESOLUTION);
	    ClauseSetInsert(store, resolvent);
	 }
	 test = ClausePosNextEqResLiteral(pos);
      }
      ClausePosFree(pos);
   }  
   return resolv_count;
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


