#!/usr/local/dist/bin/gawk -f

# Usage: generate_auto2.awk [<file>]
#
#   Given the output of find_optimal.awk, generate a selection of
#   heuristics to use and print out the C code to realize
#   this selection. Uses algorithm prefering generalists.
#
# Copyright 1999 Stephan Schulz, schulz@informatik.tu-muenchen.de

function is_empty(array)
{
   for(i in array)
   {
      return false;
   }
   return true;
}

function get_preorder(    pipe, tmp,  name, sols, time)
{
   i=0;

   pipe = "geauswert.sh protokoll*";
   while((pipe | getline tmp)>0)
   {      
      name = substr(tmp, 11, index(tmp, " ")-11);
      match(tmp, /B: [0-9]*/);
      sols = substr(tmp, RSTART+3, RLENGTH);
      match(tmp, /e: [0-9]*\.[0-9]*/);
      time = substr(tmp, RSTART+3, RLENGTH);
      global_strat_preord[name] = sprintf("%4d %6.11f", sols, 100000-time);
   }
   close(pipe);
}

function get_pid(     tmp)
{
   getline tmp < "/dev/pid";
   close("/dev/pid");
   if(!tmp)
   {
      print "sequential_eprover.awk: Cannot get PID ?!?" > "/dev/stderr";
      exit 1;
   }
   return tmp;
}

function sort_strategies(    i, pipe, tmpfile, content, pid)
{
   content = "";

   for(i in global_strat_preord)
   {
      content = content global_strat_preord[i] "," i "\n";
   }
   pid = get_pid();

   tmpfile = "__ga_tmp" pid "__";
   pipe = "sort -r > " tmpfile;
   print content | pipe;
   close(pipe);
   
   pipe = tmpfile;
   i=0;
   while((getline tmp < pipe)>0)
   {               
      tmp = substr(tmp, index(tmp, ",")+1);
      global_strat_sorted[i] = tmp;
      # print tmp;
      i++;
   }
   close(pipe);
   system("rm " tmpfile );
}

function find_best_global(   i, init, res)
{
   init = "   ";
   res = "";

   for(i in global_strat_preord)
   {
      if(global_strat_preord[i] > init)
      {
	 init = global_strat_preord[i];
	 res = i;
      }
   }
   return res;
}



function max(a,b)
{
  if(a>b)
    {
      return a;
    }
  return b;
}

function remove_strat(class, strat)
{
   # print "Strat: " strat;
   #print strategies[class];
   if(index(strategies[class], strat "|"))
   {
      # print "Remove " class;
      strat_classes[strat] = strat_classes[strat] class "|";
      delete strategies[class];
      return 1;
   }
   return 0;
}

function global_remove_strat(strat,     class, res)
{
   res = 0;
   for(class in strategies)
   {
      # print "gr: ", class, strat;
      res+=remove_strat(class, strat);
   }
   # print "Removed " res " classes for " strat;
   return res;
}


function generate_single_cond(array, letter,             res)
{
   res = "";
   if(array[letter])
   {
      res = global_prefix "Spec" array[letter] "(spec)";
      global_prefix = "&&\n       ";
   }
   return res;
}

function translate_class_to_cond(class,  res)
{
   res = "      ( /* " class " */\n";

   global_prefix = "       ";
   res = res generate_single_cond(ax_int,substr(class, 1, 1));
   res = res generate_single_cond(go_int,substr(class, 2, 1));
   res = res generate_single_cond(eq_int,substr(class, 3, 1));
   res = res generate_single_cond(ng_int,substr(class, 4, 1));
   res = res generate_single_cond(gg_int,substr(class, 5, 1));
   res = res generate_single_cond(an_int,substr(class, 6, 1));
   res = res generate_single_cond(lt_int,substr(class, 7, 1));
   res = res generate_single_cond(ts_int,substr(class, 8, 1));
   res = res generate_single_cond(pg_int,substr(class, 9, 1));
   res = res ")";

   return res;
}

function translate_class_list_to_cond(list,  res, sep, found, tmp)
{
   res = "(\n";
   sep = "";
   tmp = list;
   
   while((found = index(tmp, "|")))
   {
      res = res sep translate_class_to_cond(substr(tmp, 1, found-1));
      sep = "\n        ||\n";
      tmp = substr(tmp, found+1);
   }   
   res = res ")";

   return res;
}


BEGIN{
   class = "";
   
   ax_int["U"] = "AxiomsAreUnit";
   ax_int["H"] = "AxiomsAreHorn";
   ax_int["G"] = "AxiomsAreGeneral";
   go_int["U"] = "GoalsAreUnit";
   go_int["H"] = "GoalsAreHorn";
   go_int["G"] = "GoalsAreGeneral";
   eq_int["N"] = "NoEq";
   eq_int["S"] = "SomeEq";
   eq_int["P"] = "PureEq";
   ng_int["F"] = "FewNGPosUnits";
   ng_int["S"] = "SomeNGPosUnits";
   ng_int["M"] = "ManyNGPosUnits";
   pg_int["F"] = "FewGroundPos";
   pg_int["S"] = "SomeGroundPos";
   pg_int["M"] = "ManyGroundPos";
   gg_int["N"] = "GoalsHaveVars";
   gg_int["G"] = "GoalsAreGround";
   an_int["F"] = "FewAxioms";
   an_int["S"] = "SomeAxioms";
   an_int["M"] = "ManyAxioms";
   lt_int["F"] = "FewLiterals";
   lt_int["S"] = "SomeLiterals";
   lt_int["M"] = "ManyLiterals";
   ts_int["S"] = "SmallTerms";
   ts_int["M"] = "MediumTerms";
   ts_int["L"] = "LargeTerms";

   selstrat["NoSelection"]                    = "SelectNoLiterals";
   selstrat["NoGeneration"]                   = "SelectNoGeneration";   
   selstrat["SelectNegativeLiterals"]         = "SelectNegativeLiterals";          
   selstrat["PSelectNegativeLiterals"]        = "PSelectNegativeLiterals";       
   selstrat["SelectPureVarNegLiterals"]       = "SelectFirstVariableLiteral";      
   selstrat["PSelectPureVarNegLiterals"]      = "PSelectFirstVariableLiteral";   
   selstrat["SelectLargestNegLit"]            = "SelectLargestNegativeLiteral";    
   selstrat["PSelectLargestNegLit"]           = "PSelectLargestNegativeLiteral"; 
   selstrat["SelectSmallestNegLit"]           = "SelectSmallestNegativeLiteral";   
   selstrat["PSelectSmallestNegLit"]          = "PSelectSmallestNegativeLiteral";
   selstrat["SelectDiffNegLit"]               = "SelectDiffNegativeLiteral";       
   selstrat["PSelectDiffNegLit"]              = "PSelectDiffNegativeLiteral";    
   selstrat["SelectGroundNegLit"]             = "SelectGroundNegativeLiteral";     
   selstrat["PSelectGroundNegLit"]            = "PSelectGroundNegativeLiteral";  
   selstrat["SelectOptimalLit"]               = "SelectOptimalLiteral";            
   selstrat["PSelectOptimalLit"]              = "PSelectOptimalLiteral";         
   selstrat["SelectOptimalLit"]               = "SelectOptimalLiteral";            
   selstrat["PSelectOptimalLit"]              = "PSelectOptimalLiteral";         
   selstrat["SelectCondOptimalLit"]           = "SelectCondOptimalLiteral";            
   selstrat["PSelectCondOptimalLit"]          = "PSelectCondOptimalLiteral";         
   selstrat["SelectAllCondOptimalLit"]        = "SelectAllCondOptimalLiteral";            
   selstrat["PSelectAllCondOptimalLit"]       = "SelectAllPosCondOptimalLiteral";         
   selstrat["SelectOptimalRestrDepth2"]       = "SelectDepth2OptimalLiteral";         
   selstrat["PSelectOptimalRestrDepth2"]      = "PSelectDepth2OptimalLiteral";         
   selstrat["SelectOptimalRestrPDepth2"]      = "SelectPDepth2OptimalLiteral";         
   selstrat["PSelectOptimalRestrPDepth2"]     = "PSelectPDepth2OptimalLiteral";         
   selstrat["SelectOptimalRestrNDepth2"]      = "SelectNDepth2OptimalLiteral";         
   selstrat["PSelectOptimalRestrNDepth2"]     = "PSelectNDepth2OptimalLiteral";         
   selstrat["SelectNonRROptimalLit"]          = "SelectNonRROptimalLiteral";         
   selstrat["PSelectNonRROptimalLit"]         = "PSelectNonRROptimalLiteral";        
   selstrat["SelectNonStrongRROptimalLit"]    = "SelectNonStrongRROptimalLiteral";   
   selstrat["PSelectNonStrongRROptimalLit"]   = "PSelectNonStrongRROptimalLiteral";  
   selstrat["SelectAntiRROptimalLit"]         = "SelectAntiRROptimalLiteral";        
   selstrat["PSelectAntiRROptimalLit"]        = "PSelectAntiRROptimalLiteral";       
   selstrat["SelectNonAntiRROptimalLit"]      = "SelectNonAntiRROptimalLiteral";     
   selstrat["PSelectNonAntiRROptimalLit"]     = "PSelectNonAntiRROptimalLiteral";    
   selstrat["SelectStrongRRNonRROptimalLit"]  = "SelectStrongRRNonRROptimalLiteral"; 
   selstrat["PSelectStrongRRNonRROptimalLit"] = "PSelectStrongRRNonRROptimalLiteral";

   selstrat_old["Normal"]                = "SelectNoLiterals";
   selstrat_old["WithUnitsOnly"]         = "PSelectNegativeLiterals";
   selstrat_old["PosUnitStrategy"]       = "SelectNegativeLiterals";
   selstrat_old["NoGeneration"]          = "SelectNoGeneration";
   get_preorder();
   sort_strategies();
}
   
   

/\.\./{
   match($0, /\([0-9]+\//);
   sols = 0+substr($0, RSTART+1, RLENGTH-1);
   match($0, /\/[0-9]+\)/);
   size = 0+substr($0, RSTART+1, RLENGTH-1);   
   match($0, /\.\.\/[^\/]*\//);
   class = substr($0, RSTART+RLENGTH);
   match(class, /CLASS_[^ ]* /);
   class = substr(class, RSTART+6,RLENGTH-7);
   solutions[class] = 0;
   strategies[class] = "";
   successes[class] = sols;
   class_size[class] = size;
}

/  protokoll_/{
   solutions[class]++;
   match($0, /protokoll_.*:/);
   strategy = substr($0, RSTART+10, RLENGTH-11);
   strategies[class] = strategies[class] strategy "|";
   strat_list[strategy] += sols;
}

function get_cmd_line(file,   result)
{
   pipe =  "grep \"#.*-H'\\(.*\\)'\" " file;
   pipe | getline result;
   close(pipe);
   
   if(!result)
   {
      print "Error: Command line must specify at least '-H' directive" > "/dev/stderr";
      exit 2;
   }
   return result;
}


function translate_cmd_line(string,   key, res)
{
   res = ""

   matcher = "-W[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      if(!selstrat[key])
      {
	 print "Error: Unknown selection strategy " key > "/dev/stderr";
	 exit 2;
      }
      res = res "      control->selection_strategy = " selstrat[key] ";\n";
   }
   matcher = "--literal-selection-strategy=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      if(!selstrat[key])
      {
	 print "Error: Unknown selection strategy " key > "/dev/stderr";
	 exit 2;
      }
      res = res "      control->selection_strategy = " selstrat[key] ";\n";
   }

   matcher = "--paramod-strategy=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      if(!selstrat_old[key])
      {
	 print "Error: Unknown paramod strategy " key > "/dev/stderr";
	 exit 2;
      }
      res = res "      control->selection_strategy = " selstrat_old[key] ";\n";
   }
   
   matcher = "-F[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);      
      res = res "      control->forward_demod = " key ";\n";
   }
   matcher = "--forward_demod_level=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->forward_demod = " key ";\n";
   }

   matcher = "--selection-pos-min=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->pos_lit_sel_min = " key ";\n";
   }
   matcher = "--selection-pos-max=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->pos_lit_sel_max = " key ";\n";
   }
   matcher = "--selection-neg-min=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->neg_lit_sel_min = " key ";\n";
   }
   matcher = "--selection-neg-max=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->neg_lit_sel_max = " key ";\n";
   }
    matcher = "--selection-all-min=[^ ]*";
    mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->all_lit_sel_min = " key ";\n";
   }
   matcher = "--selection-all-max=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->all_lit_sel_max = " key ";\n";
   }  

   matcher = "--selection-weight-min=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->weight_sel_min = " key ";\n";
   }  

   matcher = "--select-on-processing-only";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   {      
      res = res "      control->select_on_proc_only = true;\n";
   }  

   return res;
}

function print_strategy(to_print)
{
   print "   {";
   print "      res = \"" gensub(/-/, "_", "g", to_print)  "\";";
   
   printf translate_cmd_line(get_cmd_line("protokoll_" to_print));
			       
   print "   }";
}

function print_definition(to_define,   data, def, newdef)
{
   pipe =  "grep \"H'(.*)\" protokoll_" to_define; 
   pipe | getline data;
   close(pipe);	 
   match(data, /-H'\(.*\)'/);
   current_class = gensub(/-/, "_", "g", to_define);
   def = substr(data, RSTART+3, RLENGTH-4);
   
   newdef = gensub(/\) *\,/, "),\\\\n\"\n\"     ", "g", def);
   
   print "\"" current_class " = \\n\"\n\"    " newdef "\\n\"";
}


END{
   max_solutions = 0;
   for(i = 0; global_strat_sorted[i]; i++)
   {
      global_remove_strat(global_strat_sorted[i]);
   }
   
   print "/* -------------------------------------------------------*/";
   print "/* The following code is generated automatically with     */";
   print "/* generate_auto.awk. Yes, it is fairly ugly ;-)          */";
   print "/* -------------------------------------------------------*/";
   print "";
   print "#ifdef CHE_PROOFCONTROL_INTERNAL";
   print "";
   print "/* Strategies used:                                       */";
   print "";

   for(i in strat_classes)
   {
      print_definition(i);
   }
   global_best = find_best_global();
   if(global_best in strat_classes)
   {
      print "/* Global best, " global_best ", already defined */";
   }
   else
   {
      print_definition(global_best);
   }
   print "#endif";
   print "";
   print "#ifdef CHE_HEURISTICS_INTERNAL";

   for(i in strat_classes)
   {
      print "   else if" translate_class_list_to_cond(strat_classes[i]);
      print_strategy(i);
   }
   print "   else /* Default strategy for this class */";      
   print_strategy(global_best);
   print "#endif";   
   print "/* -------------------------------------------------------*/";
   print "/*     End of automatically generated code.               */";
   print "/* -------------------------------------------------------*/";
}
